import { NextResponse } from "next/server"
import { prisma } from "@/lib/prisma"
import Stripe from "stripe"
import { headers } from "next/headers"

const stripe = new Stripe(process.env.STRIPE_SECRET_KEY || "", {
  apiVersion: "2023-10-16",
})

// This is your Stripe webhook secret for testing your endpoint locally
const endpointSecret = process.env.STRIPE_WEBHOOK_SECRET

export async function POST(request: Request) {
  const body = await request.text()
  const sig = headers().get("stripe-signature") as string

  let event: Stripe.Event

  try {
    event = stripe.webhooks.constructEvent(body, sig, endpointSecret!)
  } catch (err) {
    console.error(`Webhook Error: ${(err as Error).message}`)
    return NextResponse.json({ error: `Webhook Error: ${(err as Error).message}` }, { status: 400 })
  }

  // Handle the event
  switch (event.type) {
    case "checkout.session.completed":
      const session = event.data.object as Stripe.Checkout.Session

      // Retrieve the subscription details from Stripe
      const subscription = await stripe.subscriptions.retrieve(session.subscription as string)

      // Extract metadata
      const userId = Number.parseInt(session.metadata?.userId || session.client_reference_id || "0")
      const planType = session.metadata?.planType || "basic"

      if (userId) {
        // Calculate expiration date (1 year from now)
        const expirationDate = new Date()
        expirationDate.setFullYear(expirationDate.getFullYear() + 1)

        // Update the user's subscription details
        await prisma.user.update({
          where: { id: userId },
          data: {
            planType,
            expirationDate,
            // Store Stripe subscription ID for future reference
            stripeSubscriptionId: subscription.id,
          },
        })
      }
      break

    case "invoice.payment_succeeded":
      // Handle subscription renewal
      const invoice = event.data.object as Stripe.Invoice
      if (invoice.subscription) {
        const subscription = await stripe.subscriptions.retrieve(invoice.subscription as string)

        // Find user by Stripe subscription ID
        const user = await prisma.user.findFirst({
          where: { stripeSubscriptionId: subscription.id },
        })

        if (user) {
          // Extend subscription by 1 year
          const expirationDate = new Date()
          expirationDate.setFullYear(expirationDate.getFullYear() + 1)

          await prisma.user.update({
            where: { id: user.id },
            data: { expirationDate },
          })
        }
      }
      break

    case "customer.subscription.deleted":
      // Handle subscription cancellation
      const cancelledSubscription = event.data.object as Stripe.Subscription

      // Find user by Stripe subscription ID
      const user = await prisma.user.findFirst({
        where: { stripeSubscriptionId: cancelledSubscription.id },
      })

      if (user) {
        // Downgrade to free plan when subscription ends
        await prisma.user.update({
          where: { id: user.id },
          data: {
            planType: "free",
            expirationDate: null,
            stripeSubscriptionId: null,
          },
        })
      }
      break
  }

  return NextResponse.json({ received: true })
}
